@@ -40,6 +40,7 @@
 #include <string.h>
 #include <R_ext/Rdynload.h>
 #include <map>
+#include <set>
 
 
 namespace gdsfmt
@@ -218,16 +219,17 @@ COREARRAY_DLL_EXPORT void GDS_R_NodeValid_SEXP(SEXP Obj, C_BOOL ReadOrWrite)
 /// return true, if Obj is a logical object in R
 COREARRAY_DLL_EXPORT C_BOOL GDS_R_Is_Logical(PdGDSObj Obj)
 {
-	return Obj->Attribute().HasName(UTF7("R.logical"));
+	return Obj->Attribute().HasName(ASC16("R.logical"));
 }
 
 /// return true, if Obj is a factor variable
 COREARRAY_DLL_EXPORT C_BOOL GDS_R_Is_Factor(PdGDSObj Obj)
 {
-	if (Obj->Attribute().HasName(UTF7("R.class")) &&
-		Obj->Attribute().HasName(UTF7("R.levels")))
+	if (Obj->Attribute().HasName(ASC16("R.class")) &&
+		Obj->Attribute().HasName(ASC16("R.levels")))
 	{
-		return (Obj->Attribute()[UTF7("R.class")].GetStr8() == "factor");
+		return (RawText(Obj->Attribute()[ASC16("R.class")].GetStr8())
+			== "factor");
 	} else
 		return false;
 }
@@ -238,35 +240,36 @@ COREARRAY_DLL_EXPORT int GDS_R_Set_IfFactor(PdGDSObj Obj, SEXP val)
 {
 	int nProtected = 0;
 
-	if (Obj->Attribute().HasName(UTF7("R.class")) &&
-		Obj->Attribute().HasName(UTF7("R.levels")))
+	if (Obj->Attribute().HasName(ASC16("R.class")) &&
+		Obj->Attribute().HasName(ASC16("R.levels")))
 	{
-		if (Obj->Attribute()[UTF7("R.class")].GetStr8() == "factor")
+		if (RawText(Obj->Attribute()[ASC16("R.class")].GetStr8()) == "factor")
 		{
-			if (Obj->Attribute()[UTF7("R.levels")].IsArray())
+			if (Obj->Attribute()[ASC16("R.levels")].IsArray())
 			{
-				const TdsAny *p = Obj->Attribute()[UTF7("R.levels")].GetArray();
-				C_UInt32 L = Obj->Attribute()[UTF7("R.levels")].GetArrayLength();
+				const CdAny *p = Obj->Attribute()[ASC16("R.levels")].GetArray();
+				C_UInt32 L = Obj->Attribute()[ASC16("R.levels")].GetArrayLength();
 
 				SEXP levels;
 				PROTECT(levels = NEW_CHARACTER(L));
 				nProtected ++;
 				for (C_UInt32 i=0; i < L; i++)
 				{
-					SET_STRING_ELT(levels, i, mkChar(p[i].
-						GetStr8().c_str()));
+					SET_STRING_ELT(levels, i, mkCharCE(
+						RawText(p[i].GetStr8()).c_str(), CE_UTF8));
 				}
 
 				SET_LEVELS(val, levels);
 				SET_CLASS(val, mkString("factor"));
 
-			} else if (Obj->Attribute()[UTF7("R.levels")].IsString())
+			} else if (Obj->Attribute()[ASC16("R.levels")].IsString())
 			{
 				SEXP levels;
 				PROTECT(levels = NEW_CHARACTER(1));
 				nProtected ++;
-				SET_STRING_ELT(levels, 0, mkChar(Obj->Attribute()
-					[UTF7("R.levels")].GetStr8().c_str()));
+				SET_STRING_ELT(levels, 0, mkCharCE(
+					RawText(Obj->Attribute()[ASC16("R.levels")].GetStr8()).c_str(),
+					CE_UTF8));
 
 				SET_LEVELS(val, levels);
 				SET_CLASS(val, mkString("factor"));
@@ -279,15 +282,16 @@ COREARRAY_DLL_EXPORT int GDS_R_Set_IfFactor(PdGDSObj Obj, SEXP val)
 }
 
 /// return an R data object from a GDS object
-COREARRAY_DLL_EXPORT SEXP GDS_R_Array_Read(PdSequenceX Obj, C_Int32 const* Start,
-	C_Int32 const* Length, const C_BOOL *const Selection[])
+COREARRAY_DLL_EXPORT SEXP GDS_R_Array_Read(PdAbstractArray Obj,
+	C_Int32 const* Start, C_Int32 const* Length,
+	const C_BOOL *const Selection[])
 {
 	SEXP rv_ans = R_NilValue;
 	int nProtected = 0;
 
 	try
 	{
-		CdSequenceX::TSeqDimBuf St, Cnt;
+		CdAbstractArray::TArrayDim St, Cnt;
 		if (Start == NULL)
 		{
 			memset(St, 0, sizeof(St));
@@ -295,11 +299,11 @@ COREARRAY_DLL_EXPORT SEXP GDS_R_Array_Read(PdSequenceX Obj, C_Int32 const* Start
 		}
 		if (Length == NULL)
 		{
-			Obj->GetDimLen(Cnt);
+			Obj->GetDim(Cnt);
 			Length = Cnt;
 		}
 
-		CdSequenceX::TSeqDimBuf ValidCnt;
+		CdAbstractArray::TArrayDim ValidCnt;
 		Obj->GetInfoSelection(Start, Length, Selection, NULL, NULL, ValidCnt);
 
 		C_Int64 TotalCount;
@@ -318,7 +322,10 @@ COREARRAY_DLL_EXPORT SEXP GDS_R_Array_Read(PdSequenceX Obj, C_Int32 const* Start
 		{
 			#ifndef R_XLEN_T_MAX
 			if (TotalCount > TdTraits<R_xlen_t>::Max())
-				throw ErrGDSFmt("No support of long vectors, please use 64-bit R with version >=3.0!");
+			{
+				throw ErrGDSFmt(
+				"No support of long vectors, please use 64-bit R with version >=3.0!");
+			}
 			#endif
 
 			void *buffer;
@@ -364,17 +371,20 @@ COREARRAY_DLL_EXPORT SEXP GDS_R_Array_Read(PdSequenceX Obj, C_Int32 const* Start
 			if (buffer != NULL)
 			{
 				if (!Selection)
-					Obj->rData(Start, Length, buffer, SV);
+					Obj->ReadData(Start, Length, buffer, SV);
 				else
-					Obj->rDataEx(Start, Length, Selection, buffer, SV);
+					Obj->ReadDataEx(Start, Length, Selection, buffer, SV);
 			} else {
-				vector<string> strbuf(TotalCount);
+				vector<UTF8String> strbuf(TotalCount);
 				if (!Selection)
-					Obj->rData(Start, Length, &strbuf[0], SV);
+					Obj->ReadData(Start, Length, &strbuf[0], SV);
 				else
-					Obj->rDataEx(Start, Length, Selection, &strbuf[0], SV);
+					Obj->ReadDataEx(Start, Length, Selection, &strbuf[0], SV);
 				for (size_t i=0; i < strbuf.size(); i++)
-					SET_STRING_ELT(rv_ans, i, mkChar(strbuf[i].c_str()));
+				{
+					SET_STRING_ELT(rv_ans, i,
+						mkCharCE(RawText(strbuf[i]).c_str(), CE_UTF8));
+				}
 			}
 		} else {
 			if (COREARRAY_SV_INTEGER(Obj->SVType()))
@@ -417,7 +427,7 @@ COREARRAY_DLL_EXPORT SEXP GDS_R_Array_Read(PdSequenceX Obj, C_Int32 const* Start
  *  \param Func        [in] a user-defined function
  *  \param Param       [in] the parameter passed to the user-defined function
 **/
-COREARRAY_DLL_EXPORT void GDS_R_Apply(int Num, PdSequenceX ObjList[],
+COREARRAY_DLL_EXPORT void GDS_R_Apply(int Num, PdAbstractArray ObjList[],
 	int Margins[], const C_BOOL *const * const Selection[],
 	void (*InitFunc)(SEXP Argument, C_Int32 Count, PdArrayRead ReadObjList[],
 		void *_Param),
@@ -443,7 +453,7 @@ COREARRAY_DLL_EXPORT void GDS_R_Apply(int Num, PdSequenceX ObjList[],
 			throw ErrGDSFmt("GDS_R_Apply: Not numeric or character-type data.");
 		DimCnt[i] = ObjList[i]->DimCnt();
 		DLen[i].resize(DimCnt[i]);
-		ObjList[i]->GetDimLen(&(DLen[i][0]));
+		ObjList[i]->GetDim(&(DLen[i][0]));
 	}
 
 	// -----------------------------------------------------------
@@ -501,7 +511,7 @@ COREARRAY_DLL_EXPORT void GDS_R_Apply(int Num, PdSequenceX ObjList[],
 
 	// initialize buffer pointers
 	vector<void *> BufPtr(Num);
-	SEXP Func_Argument;
+	SEXP Func_Argument = R_NilValue;
 
 	if (Num > 1)
 	{
@@ -562,7 +572,7 @@ COREARRAY_DLL_EXPORT void GDS_R_Apply(int Num, PdSequenceX ObjList[],
 
 	try
 	{
-		vector<string> buffer_string;
+		vector<UTF8String> buffer_string;
 	
 		// call the initial user-defined function
 		(*InitFunc)(Func_Argument, Array[0].Count(), &ArrayList[0], Param);
@@ -580,14 +590,14 @@ COREARRAY_DLL_EXPORT void GDS_R_Apply(int Num, PdSequenceX ObjList[],
 					Array[i].Read(BufPtr[i]);
 				} else {
 					C_Int64 n = Array[i].MarginCount();
-					if (n > buffer_string.size())
+					if (n > (C_Int64)buffer_string.size())
 						buffer_string.resize(n);
 					Array[i].Read(&buffer_string[0]);
 					SEXP bufstr = (SEXP)BufPtr[i];
 					for (C_Int64 i=0; i < n; i++)
 					{
 						SET_STRING_ELT(bufstr, i,
-							mkChar(buffer_string[i].c_str()));
+							mkCharCE(RawText(buffer_string[i]).c_str(), CE_UTF8));
 					}
 				}
 			}
@@ -606,6 +616,101 @@ COREARRAY_DLL_EXPORT void GDS_R_Apply(int Num, PdSequenceX ObjList[],
 }
 
 
+struct COREARRAY_DLL_LOCAL char_ptr_less
+{
+	bool operator ()(const char *s1, const char *s2) const
+	{
+		return (strcmp(s1, s2) < 0);
+	}
+};
+
+/// is.element
+COREARRAY_DLL_EXPORT void GDS_R_Is_Element(PdAbstractArray Obj, SEXP SetEL,
+	C_BOOL Out[], size_t n_bool)
+{
+	GDS_R_NodeValid(Obj, TRUE);
+
+	R_xlen_t Len = XLENGTH(SetEL);
+	int nProtected = 0;
+	set<int> SetInt;
+	set<double> SetFloat;
+	set<const char *, char_ptr_less> SetString;
+
+	// check total number
+	C_Int64 TotalCount = Obj->TotalCount();
+	if (TotalCount != (C_Int64)n_bool)
+		throw ErrGDSFmt(".");
+
+	// determine data type
+	C_SVType ObjSV = Obj->SVType();
+	if (COREARRAY_SV_INTEGER(ObjSV))
+	{
+		PROTECT(SetEL = Rf_coerceVector(SetEL, INTSXP));
+		nProtected ++;
+		int *p = INTEGER(SetEL);
+		for (R_xlen_t i=0; i < Len; i++)
+			SetInt.insert(*p++);
+	} else if (COREARRAY_SV_FLOAT(ObjSV))
+	{
+		PROTECT(SetEL = Rf_coerceVector(SetEL, REALSXP));
+		nProtected ++;
+		double *p = REAL(SetEL);
+		for (R_xlen_t i=0; i < Len; i++)
+			SetFloat.insert(*p++);
+	} else if (COREARRAY_SV_STRING(ObjSV))
+	{
+		PROTECT(SetEL = Rf_coerceVector(SetEL, STRSXP));
+		nProtected ++;
+		for (R_xlen_t i=0; i < Len; i++)
+			SetString.insert(translateCharUTF8(STRING_ELT(SetEL, i)));
+	} else
+		throw ErrGDSFmt("Invalid SVType of array-oriented object.");
+
+	// set values
+	const int n_size = 4096;
+	C_BOOL *pL = Out;
+	CdIterator it = Obj->IterBegin();
+	
+	if (COREARRAY_SV_INTEGER(ObjSV))
+	{
+		int buffer[n_size];
+		while (TotalCount > 0)
+		{
+			int n = (TotalCount >= n_size) ? n_size : TotalCount;
+			it.ReadData(buffer, n, svInt32);
+			for (int i=0; i < n; i++, pL++)
+				*pL = SetInt.count(buffer[i]) ? TRUE : FALSE;
+			TotalCount -= n;
+		}
+	} else if (COREARRAY_SV_FLOAT(ObjSV))
+	{
+		double buffer[n_size];
+		while (TotalCount > 0)
+		{
+			int n = (TotalCount >= n_size) ? n_size : TotalCount;
+			it.ReadData(buffer, n, svFloat64);
+			for (int i=0; i < n; i++, pL++)
+				*pL = SetFloat.count(buffer[i]) ? TRUE : FALSE;
+			TotalCount -= n;
+		}
+	} else if (COREARRAY_SV_STRING(ObjSV))
+	{
+		UTF8String buffer[n_size];
+		while (TotalCount > 0)
+		{
+			int n = (TotalCount >= n_size) ? n_size : TotalCount;
+			it.ReadData(buffer, n, svStrUTF8);
+			for (int i=0; i < n; i++, pL++)
+				*pL = SetString.count(RawText(buffer[i]).c_str()) ? TRUE : FALSE;
+			TotalCount -= n;
+		}
+	}
+
+	UNPROTECT(nProtected);
+}
+
+
+
 // ===========================================================================
 // functions for file structure
 
@@ -664,7 +769,7 @@ COREARRAY_DLL_EXPORT PdGDSFile GDS_File_Open(const char *FileName,
 			for (size_t i=0; i < file->Log().List().size(); i++)
 			{
 				Msg.append(sLineBreak);
-				Msg.append(file->Log().List()[i].Msg);
+				Msg.append(RawText(file->Log().List()[i].Msg));
 			}
 		}
 		if (file) delete file;
@@ -679,7 +784,7 @@ COREARRAY_DLL_EXPORT PdGDSFile GDS_File_Open(const char *FileName,
 			for (size_t i=0; i < file->Log().List().size(); i++)
 			{
 				Msg.append(sLineBreak);
-				Msg.append(file->Log().List()[i].Msg);
+				Msg.append(RawText(file->Log().List()[i].Msg));
 			}
 		}
 		if (file) delete file;
@@ -732,9 +837,9 @@ COREARRAY_DLL_EXPORT PdGDSObj GDS_Node_Path(PdGDSFolder Node,
 	const char *Path, C_BOOL MustExist)
 {
 	if (MustExist)
-		return Node->Path(T(Path));
+		return Node->Path(ASC16(Path));
 	else
-		return Node->PathEx(T(Path));
+		return Node->PathEx(ASC16(Path));
 }
 
 
@@ -748,85 +853,87 @@ COREARRAY_DLL_EXPORT int GDS_Attr_Count(PdGDSObj Node)
 
 COREARRAY_DLL_EXPORT int GDS_Attr_Name2Index(PdGDSObj Node, const char *Name)
 {
-	return Node->Attribute().IndexName(T(Name));
+	return Node->Attribute().IndexName(ASC16(Name));
 }
 
 
 // ===========================================================================
-// functions for CdSequenceX
+// functions for CdAbstractArray
 
-COREARRAY_DLL_EXPORT int GDS_Seq_DimCnt(PdSequenceX Obj)
+COREARRAY_DLL_EXPORT int GDS_Array_DimCnt(PdAbstractArray Obj)
 {
 	return Obj->DimCnt();
 }
 
-COREARRAY_DLL_EXPORT void GDS_Seq_GetDim(PdSequenceX Obj, C_Int32 OutBuffer[],
-	size_t N_Buf)
+COREARRAY_DLL_EXPORT void GDS_Array_GetDim(PdAbstractArray Obj,
+	C_Int32 OutBuffer[], size_t N_Buf)
 {
 	if (Obj->DimCnt() > (int)N_Buf)
-		throw ErrCoreArray("Insufficient buffer in 'GDS_Seq_GetDim'.");
-	Obj->GetDimLen(OutBuffer);
+		throw ErrCoreArray("Insufficient buffer in 'GDS_Array_GetDim'.");
+	Obj->GetDim(OutBuffer);
 }
 
-COREARRAY_DLL_EXPORT C_Int64 GDS_Seq_GetTotalCount(PdSequenceX Obj)
+COREARRAY_DLL_EXPORT C_Int64 GDS_Array_GetTotalCount(PdAbstractArray Obj)
 {
 	return Obj->TotalCount();
 }
 
-COREARRAY_DLL_EXPORT enum C_SVType GDS_Seq_GetSVType(PdSequenceX Obj)
+COREARRAY_DLL_EXPORT enum C_SVType GDS_Array_GetSVType(PdAbstractArray Obj)
 {
 	return Obj->SVType();
 }
 
-COREARRAY_DLL_EXPORT unsigned GDS_Seq_GetBitOf(PdSequenceX Obj)
+COREARRAY_DLL_EXPORT unsigned GDS_Array_GetBitOf(PdAbstractArray Obj)
 {
 	return Obj->BitOf();
 }
 
-COREARRAY_DLL_EXPORT void GDS_Seq_rData(PdSequenceX Obj, C_Int32 const* Start,
-	C_Int32 const* Length, void *OutBuf, enum C_SVType OutSV)
-{
-	Obj->rData(Start, Length, OutBuf, OutSV);
-}
-
-COREARRAY_DLL_EXPORT void GDS_Seq_rDataEx(PdSequenceX Obj, C_Int32 const* Start,
-	C_Int32 const* Length, const C_BOOL *const Selection[], void *OutBuf,
+COREARRAY_DLL_EXPORT void GDS_Array_ReadData(PdAbstractArray Obj,
+	const C_Int32 *Start, const C_Int32 *Length, void *OutBuf,
 	enum C_SVType OutSV)
 {
-	Obj->rDataEx(Start, Length, Selection, OutBuf, OutSV);
+	Obj->ReadData(Start, Length, OutBuf, OutSV);
 }
 
-COREARRAY_DLL_EXPORT void GDS_Seq_wData(PdSequenceX Obj, C_Int32 const* Start,
-	C_Int32 const* Length, const void *InBuf, enum C_SVType InSV)
+COREARRAY_DLL_EXPORT void GDS_Array_ReadDataEx(PdAbstractArray Obj,
+	const C_Int32 *Start, const C_Int32 *Length,
+	const C_BOOL *const Selection[], void *OutBuf, enum C_SVType OutSV)
 {
-	Obj->wData(Start, Length, InBuf, InSV);
+	Obj->ReadDataEx(Start, Length, Selection, OutBuf, OutSV);
 }
 
-COREARRAY_DLL_EXPORT void GDS_Seq_AppendData(PdSequenceX Obj, ssize_t Cnt,
-	const void *InBuf, enum C_SVType InSV)
+COREARRAY_DLL_EXPORT void GDS_Array_WriteData(PdAbstractArray Obj,
+	const C_Int32 *Start, const C_Int32 *Length, const void *InBuf,
+	enum C_SVType InSV)
+{
+	Obj->WriteData(Start, Length, InBuf, InSV);
+}
+
+COREARRAY_DLL_EXPORT void GDS_Array_AppendData(PdAbstractArray Obj,
+	ssize_t Cnt, const void *InBuf, enum C_SVType InSV)
 {
 	Obj->Append(InBuf, Cnt, InSV);
 }
 
-COREARRAY_DLL_EXPORT void GDS_Seq_AppendString(PdSequenceX Obj,
+COREARRAY_DLL_EXPORT void GDS_Array_AppendString(PdAbstractArray Obj,
 	const char *Text)
 {
-	UTF8String val = Text;
+	UTF8String val = UTF8Text(Text);
 	Obj->Append(&val, 1, svStrUTF8);
 }
 
 
 // ===========================================================================
-// Functions for CdContainer - TdIterator
+// Functions for CdContainer - CdIterator
 
 COREARRAY_DLL_EXPORT void GDS_Iter_GetStart(PdContainer Node, PdIterator Out)
 {
-	*Out = Node->atStart();
+	*Out = Node->IterBegin();
 }
 
 COREARRAY_DLL_EXPORT void GDS_Iter_GetEnd(PdContainer Node, PdIterator Out)
 {
-	*Out = Node->atEnd();
+	*Out = Node->IterEnd();
 }
 
 COREARRAY_DLL_EXPORT PdContainer GDS_Iter_GetHandle(PdIterator I)
@@ -836,56 +943,51 @@ COREARRAY_DLL_EXPORT PdContainer GDS_Iter_GetHandle(PdIterator I)
 
 COREARRAY_DLL_EXPORT void GDS_Iter_Offset(PdIterator I, C_Int64 Offset)
 {
-	if (Offset == 1)
-		++ (*I);
-	else if (Offset == -1)
-		-- (*I);
-	else
-		(*I) += Offset;
+	*I += Offset;
 }
 
 COREARRAY_DLL_EXPORT C_Int64 GDS_Iter_GetInt(PdIterator I)
 {
-	return I->toInt();
+	return I->GetInteger();
 }
 
 COREARRAY_DLL_EXPORT C_Float64 GDS_Iter_GetFloat(PdIterator I)
 {
-	return I->toFloat();
+	return I->GetFloat();
 }
 
 COREARRAY_DLL_EXPORT void GDS_Iter_GetStr(PdIterator I, char *Out, size_t Size)
 {
-	UTF8String s = UTF16toUTF8(I->toStr());
+	string s = RawText(I->GetString());
 	if (Out)
 		strncpy(Out, s.c_str(), Size);
 }
 
 COREARRAY_DLL_EXPORT void GDS_Iter_SetInt(PdIterator I, C_Int64 Val)
 {
-	I->IntTo(Val);
+	I->SetInteger(Val);
 }
 
 COREARRAY_DLL_EXPORT void GDS_Iter_SetFloat(PdIterator I, C_Float64 Val)
 {
-	I->FloatTo(Val);
+	I->SetFloat(Val);
 }
 
 COREARRAY_DLL_EXPORT void GDS_Iter_SetStr(PdIterator I, const char *Str)
 {
-	I->StrTo(PChartoUTF16(Str));
+	I->SetString(UTF16Text(Str));
 }
 
-COREARRAY_DLL_EXPORT size_t GDS_Iter_RData(PdIterator I, void *OutBuf,
+COREARRAY_DLL_EXPORT void GDS_Iter_RData(PdIterator I, void *OutBuf,
 	size_t Cnt, enum C_SVType OutSV)
 {
-	return I->rData(OutBuf, Cnt, OutSV);
+	I->ReadData(OutBuf, Cnt, OutSV);
 }
 
-COREARRAY_DLL_EXPORT size_t GDS_Iter_WData(PdIterator I, const void *InBuf,
+COREARRAY_DLL_EXPORT void GDS_Iter_WData(PdIterator I, const void *InBuf,
 	size_t Cnt, enum C_SVType InSV)
 {
-	return I->wData(InBuf, Cnt, InSV);
+	I->WriteData(InBuf, Cnt, InSV);
 }
 
 
@@ -983,21 +1085,15 @@ COREARRAY_DLL_EXPORT void GDS_Parallel_RunThreads(
 /// functions for machine
 
 // Return the number of available CPU cores in the system
-COREARRAY_DLL_EXPORT int GDS_Mach_GetNumOfCPU()
+COREARRAY_DLL_EXPORT int GDS_Mach_GetNumOfCores()
 {
-	return Mach::GetNumberOfCPU();
+	return Mach::GetCPU_NumOfCores();
 }
 
 /// Return the size in byte of level-1 cache memory
-COREARRAY_DLL_EXPORT size_t GDS_Mach_GetL1CacheMemory()
+COREARRAY_DLL_EXPORT C_UInt64 GDS_Mach_GetCPULevelCache(int level)
 {
-	return Mach::GetL1CacheMemory();
-}
-
-/// Return the size in byte of level-2 cache memory
-COREARRAY_DLL_EXPORT size_t GDS_Mach_GetL2CacheMemory()
-{
-	return Mach::GetL2CacheMemory();
+	return Mach::GetCPU_LevelCache(level);
 }
 
 
@@ -1005,7 +1101,7 @@ COREARRAY_DLL_EXPORT size_t GDS_Mach_GetL2CacheMemory()
 // functions for reading block by block
 
 /// read an array-oriented object margin by margin
-COREARRAY_DLL_EXPORT PdArrayRead GDS_ArrayRead_Init(PdSequenceX Obj,
+COREARRAY_DLL_EXPORT PdArrayRead GDS_ArrayRead_Init(PdAbstractArray Obj,
 	int Margin, enum C_SVType SVType, const C_BOOL *const Selection[],
 	C_BOOL buf_if_need)
 {
@@ -1047,7 +1143,10 @@ COREARRAY_DLL_EXPORT void GDS_ArrayRead_BalanceBuffer(PdArrayRead array[],
 void R_init_gdsfmt(DllInfo *info)
 {
 	static const char *pkg_name = "gdsfmt";
-	#define REG(nm)    R_RegisterCCallable(pkg_name, #nm, (DL_FUNC)&nm)
+	#define REG(nm)    \
+		R_RegisterCCallable(pkg_name, #nm, (DL_FUNC)&nm)
+	#define REG2(nm, fn)    \
+		R_RegisterCCallable(pkg_name, nm, (DL_FUNC)&fn)
 
 	// R objects
 	REG(GDS_R_SEXP2Obj);
@@ -1059,6 +1158,7 @@ void R_init_gdsfmt(DllInfo *info)
 	REG(GDS_R_Set_IfFactor);
 	REG(GDS_R_Array_Read);
 	REG(GDS_R_Apply);
+	REG(GDS_R_Is_Element);
 
 	// functions for file structure
 	REG(GDS_File_Create);
@@ -1076,19 +1176,19 @@ void R_init_gdsfmt(DllInfo *info)
 	REG(GDS_Attr_Count);
 	REG(GDS_Attr_Name2Index);
 
-	// functions for CdSequenceX
-	REG(GDS_Seq_DimCnt);
-	REG(GDS_Seq_GetDim);
-	REG(GDS_Seq_GetTotalCount);
-	REG(GDS_Seq_GetSVType);
-	REG(GDS_Seq_GetBitOf);
-	REG(GDS_Seq_rData);
-	REG(GDS_Seq_rDataEx);
-	REG(GDS_Seq_wData);
-	REG(GDS_Seq_AppendData);
-	REG(GDS_Seq_AppendString);
+	// functions for CdAbstractArray
+	REG(GDS_Array_DimCnt);
+	REG(GDS_Array_GetDim);
+	REG(GDS_Array_GetTotalCount);
+	REG(GDS_Array_GetSVType);
+	REG(GDS_Array_GetBitOf);
+	REG(GDS_Array_ReadData);
+	REG(GDS_Array_ReadDataEx);
+	REG(GDS_Array_WriteData);
+	REG(GDS_Array_AppendData);
+	REG(GDS_Array_AppendString);
 
-	// functions for TdIterator
+	// functions for CdIterator
 	REG(GDS_Iter_GetStart);
 	REG(GDS_Iter_GetEnd);
 	REG(GDS_Iter_GetHandle);
@@ -1118,9 +1218,8 @@ void R_init_gdsfmt(DllInfo *info)
 	REG(GDS_Parallel_RunThreads);
 
 	// functions for machine
-	REG(GDS_Mach_GetNumOfCPU);
-	REG(GDS_Mach_GetL1CacheMemory);
-	REG(GDS_Mach_GetL2CacheMemory);
+	REG(GDS_Mach_GetNumOfCores);
+	REG(GDS_Mach_GetCPULevelCache);
 
 	// functions for reading block by block
 	REG(GDS_ArrayRead_Init);
@@ -1128,6 +1227,19 @@ void R_init_gdsfmt(DllInfo *info)
 	REG(GDS_ArrayRead_Read);
 	REG(GDS_ArrayRead_Eof);
 	REG(GDS_ArrayRead_BalanceBuffer);
+
+
+	/// compatible with <= gdsfmt_1.0.5
+	REG2("GDS_Seq_DimCnt", GDS_Array_DimCnt);
+	REG2("GDS_Seq_GetDim", GDS_Array_GetDim);
+	REG2("GDS_Seq_GetTotalCount", GDS_Array_GetTotalCount);
+	REG2("GDS_Seq_GetSVType", GDS_Array_GetSVType);
+	REG2("GDS_Seq_GetBitOf", GDS_Array_GetBitOf);
+	REG2("GDS_Seq_rData", GDS_Array_ReadData);
+	REG2("GDS_Seq_rDataEx", GDS_Array_ReadDataEx);
+	REG2("GDS_Seq_wData", GDS_Array_WriteData);
+	REG2("GDS_Seq_AppendData", GDS_Array_AppendData);
+	REG2("GDS_Seq_AppendString", GDS_Array_AppendString);
 }
 
 } // extern "C"
