@@ -6,7 +6,7 @@
 // _/_/_/   _/_/_/  _/_/_/_/_/     _/     _/_/_/   _/_/
 // ===========================================================
 //
-// dBitGDS.cpp: CoreArray Containers for extended types
+// dBitGDS.cpp: Bit operators and classes of GDS format
 //
 // Copyright (C) 2007 - 2014	Xiuwen Zheng
 //
@@ -30,8 +30,8 @@
 
 namespace CoreArray
 {
-	#ifdef COREARRAY_SUNPROCC
-		// silly compiler
+	#ifdef COREARRAY_CC_SUNPRO
+		// specific compiler
 		#define PREFIX	extern
 	#else
 		#define PREFIX
@@ -85,47 +85,16 @@ namespace CoreArray
 }
 
 
-
 using namespace std;
 using namespace CoreArray;
 
-// bit operator
 
-COREARRAY_DLL_DEFAULT void CoreArray::bitClear(
-	TdAllocator &alloc, SIZE64 p, C_Int64 Len)
-{
-	C_UInt8 k, kEnd, B;
-	if (Len > 0)
-	{
-		// Head
-		k = ((C_UInt8)p) & 0x07; p = p >> 3;
-		if (k > 0)
-		{
-			B = alloc.r8(p);
-			kEnd = (7 < k+(ssize_t)Len-1) ? 7 : (k+(ssize_t)Len-1);
-			for (; k <= kEnd; k++, Len--)
-				B = B & CoreArray_MaskBit1ArrayNot[k];
-			alloc.w8(p, B); p++;
-		}
-		// Middle
-		if (Len >= 8)
-		{
-			B = ((C_UInt8)Len) & 0x07; Len = Len >> 3;
-			alloc.Fill(p, Len, 0);
-			p += Len; Len = B;
-		}
-		// End
-		if (Len > 0)
-		{
-			B = alloc.r8(p);
-			B = B & (0xFF << ((C_UInt8)Len));
-			alloc.w8(p, B);
-		}
-	}
-}
+// =====================================================================
+// Bit operators
+// =====================================================================
 
-COREARRAY_DLL_DEFAULT void CoreArray::bitBinShr(
-	void *Buf, size_t NByte, C_UInt8 NShr)
+COREARRAY_DLL_DEFAULT void CoreArray::BitBinShr(void *Buf,
+	size_t NByte, C_UInt8 NShr)
 {
 	C_UInt32 *p32, *p32a, D32;
 	C_UInt8 *p8, *p8a, D8, xNShr, xNShr8;
@@ -154,8 +123,8 @@ COREARRAY_DLL_DEFAULT void CoreArray::bitBinShr(
 	}
 }
 
-COREARRAY_DLL_DEFAULT void CoreArray::bitBinShl(
-	void *Buf, size_t NByte, C_UInt8 NShl)
+COREARRAY_DLL_DEFAULT void CoreArray::BitBinShl(void *Buf,
+	size_t NByte, C_UInt8 NShl)
 {
 	C_UInt32 *p32, D32a, D32;
 	C_UInt8 *p8, D8a, D8, xNShl, xNShl8;
@@ -165,77 +134,123 @@ COREARRAY_DLL_DEFAULT void CoreArray::bitBinShl(
 	xNShl = 32 - NShl; xNShl8 = 8 - NShl;
 
 	p32 = (C_UInt32*)Buf; D32a = 0;
-	while (NByte >= 4) {
+	while (NByte >= 4)
+	{
 		D32 = *p32; *p32 = (D32 << NShl) | D32a;
 		D32a = D32 >> xNShl;
 		++p32; NByte -=4;
 	}
 
 	p8 = (C_UInt8*)p32; D8a = D32a;
-	while (NByte > 0) {
+	while (NByte > 0)
+	{
 		D8 = *p8; *p8 = (D8 << NShl) | D8a;
 		D8a = D8 >> xNShl8;
 		++p8; --NByte;
 	}
 }
 
+COREARRAY_DLL_DEFAULT void CoreArray::BitClear(CdAllocator &alloc,
+	SIZE64 p, SIZE64 Len)
+{
+	C_UInt8 k, kEnd, B;
+	if (Len > 0)
+	{
+		// Head
+		k = ((C_UInt8)p) & 0x07; p = p >> 3;
+		alloc.SetPosition(p);
+		if (k > 0)
+		{
+			B = alloc.R8b();
+			kEnd = (7 < k+(ssize_t)Len-1) ? 7 : (k+(ssize_t)Len-1);
+			for (; k <= kEnd; k++, Len--)
+				B = B & CoreArray_MaskBit1ArrayNot[k];
+			alloc.SetPosition(p);
+			alloc.W8b(B);
+		}
+		// Middle
+		if (Len >= 8)
+		{
+			B = ((C_UInt8)Len) & 0x07;
+			Len = Len >> 3;
+			alloc.ZeroFill(Len);
+			p += Len; Len = B;
+		}
+		// End
+		if (Len > 0)
+		{
+			B = alloc.R8b();
+			B = B & (0xFF << ((C_UInt8)Len));
+			alloc.SetPosition(alloc.Position()-1);
+			alloc.W8b(B);
+		}
+	}
+}
+
 COREARRAY_INLINE static C_UInt8 xb(C_UInt8 v)
 {
 	return v & 0x07;
 }
 
-static size_t bitCpyToBuf(TdAllocator &alloc, const SIZE64 pS,
-	void *Buf, size_t L)
+static size_t bit_cpy_to_buf(CdAllocator &alloc, const SIZE64 pS,
+	void *Buffer, size_t L)
 {
-	SIZE64 ppS, p;
-
-	ppS = pS >> 3; p = pS + L;
+	SIZE64 ppS = pS >> 3;
+	SIZE64 p = pS + L;
 	L = (xb(p) == 0) ? ((p >> 3) - ppS) : ((p >> 3) - ppS + 1);
-	alloc.Read(ppS, Buf, L);
+	alloc.SetPosition(ppS);
+	alloc.ReadData(Buffer, L);
 	return L;
 }
 
-COREARRAY_DLL_DEFAULT void CoreArray::bitBufToCpy(
-	TdAllocator &alloc, SIZE64 pD, void *Buf, size_t L)
+COREARRAY_DLL_DEFAULT void CoreArray::BitBufToCpy(CdAllocator &alloc,
+	SIZE64 pD, void *Buffer, size_t L)
 {
 	C_UInt8 *pB, i, B, xpD, xpDL;
 	SIZE64 p;
 
-	pB = (C_UInt8*)Buf; xpD = xb(pD);
+	pB = (C_UInt8*)Buffer; xpD = xb(pD);
+	p = pD >> 3;
+	alloc.SetPosition(p);
 	if (xpD > 0)
 	{
-		p = pD >> 3;
-		B = alloc.r8(p);
+		B = alloc.R8b();
     	xpDL = ((xpD+L-1) < 7) ? (xpD+L-1) : 7;
 		for (i = xpD; i <= xpDL; i++)
 		{
-			B = (B & CoreArray_MaskBit1ArrayNot[i]) | (*pB & CoreArray_MaskBit1Array[i]);
+			B = (B & CoreArray_MaskBit1ArrayNot[i]) |
+				(*pB & CoreArray_MaskBit1Array[i]);
 			--L; pD++;
 		}
-		alloc.w8(p, B); ++pB;
+		alloc.SetPosition(p);
+		alloc.W8b(B);
+		++ pB;
 	}
 
-	pD >>= 3;
 	if (L >= 8)
 	{
 		B = L & 0x07; L >>= 3;
-		alloc.Write(pD, (void*)pB, L);
+		alloc.WriteData((void*)pB, L);
 		pB += L; pD += L; L = B;
 	}
 
 	if (L > 0)
 	{
-		B = alloc.r8(pD);
+		B = alloc.R8b();
 		for (i = 0; (size_t)i < L; i++)
-			B = (B & CoreArray_MaskBit1ArrayNot[i]) | (*pB & CoreArray_MaskBit1Array[i]);
-		alloc.w8(pD, B);
+		{
+			B = (B & CoreArray_MaskBit1ArrayNot[i]) |
+				(*pB & CoreArray_MaskBit1Array[i]);
+		}
+		alloc.SetPosition(alloc.Position() - 1);
+		alloc.W8b(B);
 	}
 }
 
-COREARRAY_DLL_DEFAULT void CoreArray::bitMoveBits(
-	TdAllocator &alloc, SIZE64 pS, SIZE64 pD, SIZE64 Len)
+COREARRAY_DLL_DEFAULT void CoreArray::BitMoveBits(CdAllocator &alloc,
+	SIZE64 pS, SIZE64 pD, SIZE64 Len)
 {
-	C_UInt8 Buf[65536];
+	C_UInt8 Buf[COREARRAY_STREAM_BUFFER];
 	size_t L, LD;
 
 	if (pS < pD)
@@ -245,25 +260,52 @@ COREARRAY_DLL_DEFAULT void CoreArray::bitMoveBits(
 		{
 			L = (Len <= (SIZE64)(sizeof(Buf)*8-16)) ? Len : (sizeof(Buf)*8-16);
 			pS -= L; pD -= L; Len -= L;
-			LD = bitCpyToBuf(alloc, pS, Buf, L);
+			LD = bit_cpy_to_buf(alloc, pS, Buf, L);
 			if (xb(pS) < xb(pD))
-				bitBinShl((void*)Buf, LD+1, xb(pD) - xb(pS));
+				BitBinShl((void*)Buf, LD+1, xb(pD) - xb(pS));
 			else if (xb(pS) > xb(pD))
-				bitBinShr((void*)Buf, LD, xb(pS) - xb(pD));
-			bitBufToCpy(alloc, pD, Buf, L);
+				BitBinShr((void*)Buf, LD, xb(pS) - xb(pD));
+			BitBufToCpy(alloc, pD, Buf, L);
 		}
 	} else if (pS > pD)
 	{
 		while (Len > 0)
 		{
 			L = (Len <= (SIZE64)(sizeof(Buf)*8-16)) ? Len : (sizeof(Buf)*8-16);
-			LD = bitCpyToBuf(alloc, pS, Buf, L);
+			LD = bit_cpy_to_buf(alloc, pS, Buf, L);
 			if (xb(pS) < xb(pD))
-				bitBinShl((void*)Buf, LD+1, xb(pD) - xb(pS));
+				BitBinShl((void*)Buf, LD+1, xb(pD) - xb(pS));
 			else if (xb(pS) > xb(pD))
-				bitBinShr((void*)Buf, LD, xb(pS) - xb(pD));
-			bitBufToCpy(alloc, pD, Buf, L);
+				BitBinShr((void*)Buf, LD, xb(pS) - xb(pD));
+			BitBufToCpy(alloc, pD, Buf, L);
 			pS += L; pD += L; Len -= L;
 		}
 	}
 }
+
+COREARRAY_DLL_DEFAULT C_Int32 CoreArray::BitSet_IfSigned(C_Int32 val,
+	unsigned nbit)
+{
+	static const C_UInt32 BitFlag[33] = {
+		0, 0x1, 0x2, 0x4, 0x8,
+		0x10, 0x20, 0x40, 0x80,
+		0x100, 0x200, 0x400, 0x800,
+		0x1000, 0x2000, 0x4000, 0x8000,
+		0x10000, 0x20000, 0x40000, 0x80000,
+		0x100000, 0x200000, 0x400000, 0x800000,
+		0x1000000, 0x2000000, 0x4000000, 0x8000000,
+		0x10000000, 0x20000000, 0x40000000, 0x80000000 };
+	static const C_UInt32 BitNeg[33] = {
+		0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFC, 0xFFFFFFF8, 0xFFFFFFF0,
+		0xFFFFFFE0, 0xFFFFFFC0, 0xFFFFFF80, 0xFFFFFF00,
+		0xFFFFFE00, 0xFFFFFC00, 0xFFFFF800, 0xFFFFF000,
+		0xFFFFE000, 0xFFFFC000, 0xFFFF8000, 0xFFFF0000,
+		0xFFFE0000, 0xFFFC0000, 0xFFF80000, 0xFFF00000,
+		0xFFE00000, 0xFFC00000, 0xFF800000, 0xFF000000,
+		0xFE000000, 0xFC000000, 0xF8000000, 0xF0000000,
+		0xE0000000, 0xC0000000, 0x80000000, 0x00000000 };
+
+	if (val & BitFlag[nbit]) val |= BitNeg[nbit];
+
+	return val;
+}
