@@ -40,6 +40,8 @@
 #define _HEADER_PARALLEL_
 
 #include <dPlatform.h>
+#include <dTrait.h>
+
 #include <vector>
 #include <memory>
 #include <algorithm>
@@ -81,6 +83,7 @@ namespace CoreArray
 		COREARRAY_INLINE int Percent() const { return fPercent; }
 		/// Return the total number
 		COREARRAY_INLINE C_Int64 Total() const { return fTotal; }
+
 	protected:
 		TPercentMode fMode;
 		C_Int64 fTotal, vProg[101], vCurrent, *vptrProg;
@@ -99,7 +102,7 @@ namespace CoreArray
 	namespace Parallel
 	{
 		/// Exceptions for conversion
-		class ErrParallel: public ErrCoreArray
+		class COREARRAY_DLL_EXPORT ErrParallel: public ErrCoreArray
 		{
 		public:
 			ErrParallel(): ErrCoreArray()
@@ -135,7 +138,7 @@ namespace CoreArray
 
 		namespace _INTERNAL
 		{
-			template<class TCLASS> struct _pThreadStructEx
+			template<class TCLASS> struct COREARRAY_DLL_DEFAULT _pThreadStructEx
 			{
 				TCLASS * obj;
 				void (TCLASS::*proc)(CdThread *, int);
@@ -143,7 +146,7 @@ namespace CoreArray
 				CParallelBase *cpBase;
 			};
 
-			template<class TCLASS>
+			template<class TCLASS> COREARRAY_DLL_DEFAULT
 				int _pDoThreadEx(CdThread *Thread, _pThreadStructEx<TCLASS> Data)
 			{
 				Data.cpBase->InitThread();
@@ -551,11 +554,11 @@ namespace CoreArray
 				if (BufSize > TotalSize) BufSize = TotalSize;
 				if (fProgress) fProgress->Init(TotalSize);
 				// Initialize
-				std::auto_ptr<OUTTYPE> Buffer(new OUTTYPE[BufSize]);
+				std::vector<OUTTYPE> Buffer(BufSize);
 				_IStruct<TCLASS, TINDEX, OUTTYPE> Rec;
 				Rec.Obj = Obj;
 				Rec.Proc = Proc; Rec.QueueFunc = QueueFunc;
-				Rec.Buffer = Buffer.get();
+				Rec.Buffer = &(Buffer[0]);
 				Rec.Idx = Rec.IdxBase = StartIndex;
 				Rec.TotalSize = TotalSize;
 				Rec.BufSize = Rec.IndexEnd = BufSize;
@@ -583,11 +586,11 @@ namespace CoreArray
 				if (BufSize > TotalSize) BufSize = TotalSize;
 				if (fProgress) fProgress->Init(TotalSize);
 				// Initialize
-				std::auto_ptr<OUTTYPE> Buffer(new OUTTYPE[BufSize]);
+				std::vector<OUTTYPE> Buffer(BufSize);
 				_IStruct2<TCLASS, TINDEX, OUTTYPE, THREADDATA> Rec;
 				Rec.Obj = Obj;
 				Rec.Proc = Proc; Rec.QueueFunc = QueueFunc; Rec.InternalFunc = InternalFunc;
-				Rec.Buffer = Buffer.get();
+				Rec.Buffer = &(Buffer[0]);
 				Rec.Idx = Rec.IdxBase = StartIndex;
 				Rec.TotalSize = TotalSize;
 				Rec.BufSize = Rec.IndexEnd = BufSize;
@@ -614,11 +617,11 @@ namespace CoreArray
 				if (BufSize > TotalSize) BufSize = TotalSize;
 				if (fProgress) fProgress->Init(TotalSize);
 				// Initialize
-				std::auto_ptr<OUTTYPE> Buffer(new OUTTYPE[BufSize]);
+				std::vector<OUTTYPE> Buffer(BufSize);
 				_IStructEx<TCLASS, TINDEX, OUTTYPE> Rec;
 				Rec.Obj = Obj;
 				Rec.Proc = Proc; Rec.QueueFunc = QueueFunc;
-				Rec.Buffer = Buffer.get();
+				Rec.Buffer = &(Buffer[0]);
 				Rec.Idx = Rec.IdxBase = StartIndex;
 				Rec.TotalSize = TotalSize;
 				Rec.BufSize = Rec.IndexEnd = BufSize;
@@ -828,11 +831,11 @@ namespace CoreArray
 				if (SubBufSize > BufSize) SubBufSize = BufSize;
 				if (fProgress) fProgress->Init(TotalSize);
 				// Initialize
-				std::auto_ptr<OUTTYPE> Buffer(new OUTTYPE[BufSize]);
+				std::vector<OUTTYPE> Buffer(BufSize);
 				_IStruct<TCLASS, TINDEX, OUTTYPE> Rec;
 				Rec.Obj = Obj;
 				Rec.Proc = Proc; Rec.QueueFunc = QueueFunc;
-				Rec.Buffer = Buffer.get();
+				Rec.Buffer = &(Buffer[0]);
 				Rec.Idx = Rec.IdxBase = StartIndex;
 				Rec.TotalSize = TotalSize;
 				Rec.BufSize = Rec.IndexEnd = BufSize;
@@ -861,11 +864,11 @@ namespace CoreArray
 				if (BufSize > TotalSize) BufSize = TotalSize;
 				if (fProgress) fProgress->Init(TotalSize);
 				// Initialize
-				std::auto_ptr<OUTTYPE> Buffer(new OUTTYPE[BufSize]);
+				std::vector<OUTTYPE> Buffer(BufSize);
 				_IStruct2<TCLASS, TINDEX, OUTTYPE, THREADDATA> Rec;
 				Rec.Obj = Obj;
 				Rec.Proc = Proc; Rec.QueueFunc = QueueFunc; Rec.InternalFunc = InternalFunc;
-				Rec.Buffer = Buffer.get();
+				Rec.Buffer = &(Buffer[0]);
 				Rec.Idx = Rec.IdxBase = StartIndex;
 				Rec.TotalSize = TotalSize;
 				Rec.BufSize = Rec.IndexEnd = BufSize;
